/*-------------------------------------------------------------------------+
| Copyright 2022 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.ext.reuse.service;

import static org.eclipse.emf.ecore.util.EcoreUtil.replace;
import static org.fortiss.tooling.kernel.utils.EcoreUtils.replaceEObjectReferences;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.layout.ILayoutData;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;

/***
 * Implemented basic {@link IReuseProvider} based on layouted {@link EObject}
 * aka {@link ILayoutedModelElement}. Can be extended or overridden by more
 * specific implemented {@link IReuseProvider}.
 *
 * @author bergemann
 *
 * @param <T> The subclass of {@link EObject} for which this provider base shall
 *            be extended.
 */
public class LayoutedReuseProviderBase<T extends ILayoutedModelElement> extends ReuseProviderBase<T> {

	/** {@inheritDoc} */
	@Override
	public boolean updateReuseElement(T elementToBeUpdated, T newElement) {
		// keep old element position (replace it in new element)
		ILayoutData oldPosLayout = null;
		for (ILayoutData layout : elementToBeUpdated.getLayoutData()) {
			if (layout.getKey().equals("pos")) {
				oldPosLayout = layout;
			}
		}
		if (oldPosLayout != null) {
			for (ILayoutData layout : newElement.getLayoutData()) {
				if (layout.getKey().equals("pos")) {
					replace(layout, oldPosLayout);
				}
			}
		}

		replaceEObjectReferences(elementToBeUpdated, newElement);

		replace(elementToBeUpdated, newElement);

		return true;
	}

}
